<?php

/**
 * @file
 * Contains the list style plugin.
 */

/**
 * Style plugin to render Gantt charts.
 *
 * @ingroup views_style_plugins
 */
class views_gantt_plugin_style_gantt extends views_plugin_style_list {
  private $option_fields;
  private $options_node;

  function option_definition() {
    $options = parent::option_definition();

    $options['class'] = array('default' => '');
    $options['wrapper_class'] = array('default' => 'item-list');
    $options['id_field'] = array('default' => '');
    $options['name_field'] = array('default' => '');
    $options['parent_field'] = array('default' => '');
    $options['date_field'] = array('default' => '');
    $options['duration_field'] = array('default' => '');
    $options['progress_field'] = array('default' => '');
    $options['project_id_field'] = array('default' => '');
    $options['project_date_field'] = array('default' => '');
    $options['parent_id_field'] = array('default' => '');
    $options['predecessor_id_field'] = array('default' => '');
    return $options;
  }
 
  function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);

    $fields = array('' => t('<None>'));

    foreach ($this->display->handler->get_handlers('field') as $field => $handler) {
      if ($label = $handler->label()) {
        $fields[$field] = $label;
      }
      else {
        $fields[$field] = $handler->ui_name();
      }
    }
  
    $form['id_field'] = array(
      '#type' => 'select',
      '#title' => t('ID field'),
      '#options' => $fields,
      '#default_value' => $this->options['id_field'],
      '#description' => t('Select the field that contains nid of each record.'),
      '#required' => TRUE,
    );
    $form['name_field'] = array(
      '#type' => 'select',
      '#title' => t('Name field'),
      '#options' => $fields,
      '#default_value' => $this->options['name_field'],
      '#description' => t('Select the field that contains name of each record.'),
      '#required' => TRUE,
    );    
    $form['date_field'] = array(
      '#type' => 'select',
      '#title' => t('Date field'),
      '#options' => $fields,
      '#default_value' => $this->options['date_field'],
      '#description' => t('Select the field that contains the date (timestamp or valid <a href="http://www.php.net/manual/en/datetime.formats.php" target="_blank">date format</a>) of the node in the selected row.'),
      '#required' => true,
    );
    $form['duration_field'] = array(
      '#type' => 'select',
      '#title' => t('Duration field'),
      '#options' => $fields,
      '#default_value' => $this->options['duration_field'],
      '#description' => t('Select the field that contains the duration (in hours) of the node in the selected row.'),
      '#required' => true,
    );
    $form['progress_field'] = array(
      '#type' => 'select',
      '#title' => t('Progress field'),
      '#options' => $fields,
      '#default_value' => $this->options['progress_field'],
      '#description' => t('Select the field that contains the progress of the node in percents.'),
      '#required' => true,
    ); 
    $form['project_id_field'] = array(
      '#type' => 'select',
      '#title' => t('Project ID field'),
      '#options' => $fields,
      '#default_value' => $this->options['project_id_field'],
      '#description' => t('Select the field that contains nid of the record\'s project node.'),
      '#required' => true,
    ); 
    $form['project_date_field'] = array(
      '#type' => 'select',
      '#title' => t('Project date field'),
      '#options' => $fields,
      '#default_value' => $this->options['project_date_field'],
      '#description' => t('Select the field that contains the date (timestamp or valid <a href="http://www.php.net/manual/en/datetime.formats.php" target="_blank">date format</a>) of the record\'s project node.'),
      '#required' => true,
    );        
    $form['parent_id_field'] = array(
      '#type' => 'select',
      '#title' => t('Parent ID field'),
      '#options' => $fields,
      '#default_value' => $this->options['parent_id_field'],
      '#description' => t('Select the field that contains nid of the record\'s parent node.'),
    ); 
    $form['predecessor_id_field'] = array(
      '#type' => 'select',
      '#title' => t('Predecessor ID field'),
      '#options' => $fields,
      '#default_value' => $this->options['predecessor_id_field'],
      '#description' => t('Select the field that contains nid of the record\'s predecessor node.'),
    );              
  }

  function render() {
    // Add css to fix chart style
    drupal_add_css(drupal_get_path('module', 'views_gantt') . "/css/reset.css");
    // Add js
    drupal_add_js(drupal_get_path('module', 'views_gantt') . "/js/views_gantt.js");
    // Add dhtmlxgantt library
    libraries_load('dhtmlxgantt');
    // Add jquery ui Dialog library
    drupal_add_library('system', 'ui.dialog');

    // Get real names of fields that we put in style settings
    // If field come from Fields API, we get it's name 
    // Otherwise (if field is 'node_title' for example) we get field alias
    $this->option_fields = array();
    $this->option_fields_node = array();
    $view_fields = $this->view->field;
    foreach ($this->view->display_handler->get_handlers('field') as $key => $handler) {
      if (isset($view_fields[$key]) && isset($handler->field_info)) {
        $this->option_fields[$key] = 'field_' . $key;
      } else {
        $this->option_fields[$key] = $handler->field_alias;
      }
    }

    // Some fields in $this->options can have incorrect names (e.g. field_date_1)
    // We need to create new array which contains correct field names to use it in
    // views_gantt_get_node_field_value() function
    foreach ($this->options as $key => $value) {
      if (is_string($value) && isset($view_fields[$value])) {
        $this->options_node[$key] = $view_fields[$value]->field;
      }
    }

    // Add globals variables to use it in js
    $project_id = $this->views_gantt_get_field_value($this->view->result[0], 'project_id_field');
    $settings = array(
      'views_gantt' => array(
        'view_name' => $this->view->name,
        'project_id' => $project_id,
      ),
    );
    drupal_add_js($settings, 'setting');    

    // Get project data
    $node = node_load($project_id);
    $project_date = $this->views_gantt_get_field_value($this->view->result[0], 'project_date_field');
    $project = array(
      'name' => $node->title,
      'id' => $node->nid,
      'startdate' => $project_date,
    );

    // Get array of tasks
    $tasks = array();
    foreach ($this->view->result as $row) {
      $tasks[$this->views_gantt_get_field_value($row, 'id_field')] = array(
        'name' => $this->views_gantt_get_field_value($row, 'name_field'),
        'est' => $this->views_gantt_get_field_value($row, 'date_field'),
        'duration' => $this->views_gantt_get_field_value($row, 'duration_field'),
        'percentcompleted' => $this->views_gantt_get_field_value($row, 'progress_field'),
        'predecessortasks' => $this->views_gantt_get_field_value($row, 'predecessor_id_field'),
        'parent_id' => $this->views_gantt_get_field_value($row, 'parent_id_field'),
        'childtasks' => '',
      );
    }

    // If we have parent or predecessor tasks that doesn't match the filter,
    // we need to get them to avoid gantt chart errors
    foreach ($tasks as $task_id => $task) {
      if ($task['predecessortasks'] && !array_key_exists($task['predecessortasks'], $tasks)) {
        $this->views_gantt_get_missing_task($task['predecessortasks'], $tasks, $task_id);
      }
      if ($task['parent_id'] && !array_key_exists($task['parent_id'], $tasks)) {
        $this->views_gantt_get_missing_task($task['parent_id'], $tasks, $task_id);
      }      
    }
    // Check if there are any incorrect tasks in array and remove them
    foreach ($tasks as $task_id => $task) {
      $is_incorrect = ($task['predecessortasks'] && !array_key_exists($task['predecessortasks'], $tasks))
        || ($task['parent_id'] && !array_key_exists($task['parent_id'], $tasks));
      if ($is_incorrect) {
        unset($tasks[$task_id]);
      }
    }

    // Save data in session to use it when we will build XML file for chart
    $_SESSION['views_gantt']['project'] = $project;
    $_SESSION['views_gantt']['tasks'] = $tasks;

    return parent::render();
  }

  /**
   * Returns value of specific field
   * 
   * @param object $row Node object from view result array
   * @param string $field_options_name 
   *  Option key from views_gantt_plugin_style_gantt::option_definition()
   * 
   * @return string Field value from node object
   */
  function views_gantt_get_field_value($row, $field_options_name) {
    $field_name = $this->options[$field_options_name];
    $field_value = '';
    
    // If field value is array, we try to get it's raw value,
    // if it's not possible, we get rendered value
    if (isset($this->option_fields[$field_name]) && !empty($this->option_fields[$field_name])) {
      $field_value = $row->{$this->option_fields[$field_name]};    
      if (is_array($field_value)) {
        if (isset($field_value[0]['raw']['value'])) {
          $field_value = $field_value[0]['raw']['value'];
        } else if (isset($field_value[0]['rendered']['#markup'])) {
          $field_value = $field_value[0]['rendered']['#markup'];
        }       
      }

      // If field should provide date, we need to return it's value in specific date format (Y,m,j)
      if (!is_array($field_value) && ($field_options_name == 'date_field' || $field_options_name == 'project_date_field')) {       
        views_gantt_normalize_date_field($field_value);
      }      
    }

    return $field_value;
  }

  /**
   * Get tasks that have been excluded by exposed filters,
   * but must be included to XML for correct Gantt chart.
   */
  function views_gantt_get_missing_task($task_to_load, &$tasks, $task_id) {
    $fields = $this->options;
    $node = node_load($task_to_load);
    if ($node) {
      $loaded_task = array(
        'name' => $this->views_gantt_get_node_field_value($node, 'name_field'),
        'est' => $this->views_gantt_get_node_field_value($node, 'date_field'),
        'duration' => $this->views_gantt_get_node_field_value($node, 'duration_field'),
        'percentcompleted' => $this->views_gantt_get_node_field_value($node, 'progress_field'),
        'predecessortasks' => $this->views_gantt_get_node_field_value($node, 'predecessor_id_field'),
        'parent_id' => $this->views_gantt_get_node_field_value($node, 'parent_id_field'),
        'project_id' => $this->views_gantt_get_node_field_value($node, 'project_id_field'),
        'childtasks' => '',        
      );

      // If parent/predecessor is incorrect (don't have date or duration),
      // we don't include it and also exclude it's child/successor
      if (!$loaded_task['est'] || !$loaded_task['duration']) {
        unset($tasks[$task_id]);
      } else {
        $tasks[$task_to_load] = $loaded_task;

        // If received task has parent or predecessor, we need to get them too
        if ($loaded_task['predecessortasks'] && !array_key_exists($loaded_task['predecessortasks'], $tasks)) {
          $this->views_gantt_get_missing_task($loaded_task['predecessortasks'], $tasks, $task_to_load);
        }
        if ($loaded_task['parent_id'] && !array_key_exists($loaded_task['parent_id'], $tasks)) {
          $this->views_gantt_get_missing_task($loaded_task['parent_id'], $tasks, $task_to_load);
        } 
      }     
    }
  }

  /**
   * Returns value of specific field from node
   * 
   * @param object $row Node object received via node_load()
   * @param string $field_options_name 
   *  Option key from views_gantt_plugin_style_gantt::option_definition()
   * 
   * @return string Field value from node object
   */
  function views_gantt_get_node_field_value($node, $field_options_name) {
    $field_name = $this->options_node[$field_options_name];
    $field_value = '';
    $lang = LANGUAGE_NONE;

    if (isset($node->$field_name) && !empty($node->$field_name)) {
      $field_value = $node->$field_name;
      if (is_array($field_value)) {
        // Get field info
        $field_info =  field_info_field($field_name);

        // If field is translatable, we check if it has an index equals to node language
        $is_translatable = field_is_translatable('node', $field_info);
        if ($is_translatable && isset($node->{$field_name}[$node->language])) {
          $lang = $node->language;
        }

        $value_keys = array_keys($field_info['columns']);
        $field_value = $node->{$field_name}[$lang][0][$value_keys[0]];         
      }

      // If field should provide date, we need to return it's value in specific date format (Y,m,j)  
      if (!is_array($field_value) && ($field_options_name == 'date_field' || $field_options_name == 'project_date_field')) {
        views_gantt_normalize_date_field($field_value);
      }      
    } 

    return $field_value;    
  }
}
